// ***********************
// ** zx81keyboard.h    **
// ** Class Definitions **
// ** For ZX81 keyboard **
// ***********************

enum KEYMODES {EMULATOR, STANDARD};
enum KEYSTATES {NORMAL, NORMAL_SHIFTED, GRAPHICS, GRAPHICS_SHIFTED, FUNCTION};

#define NUM_ROWS 8
#define NUM_COLS 5 

#define SHIFT_COL 4
#define SHIFT_ROW 5
//#define FUNCTION_COL 4
//#define FUNCTION_ROW 6
//#define GRAPHICS_COL 3
//#define GRAPHICS_ROW 2

#define DEBOUNCE_DELAY 200 //debounce countdown value.
#define DEBOUNCE_DELAY_REPEAT 600

// Defines a single key and its 4 possible Major KEYSTATES: NORMAL, NORMAL_SHIFTED, GRAPHICS, GRAPHICS_SHIFTED
// Returns 5 KEYSTATES, adds FUNCTION: Caps Lock).
class ZxKey  {
	private:
		// Standard Keyboard Characters for Normal PC Use
		byte bNormal;			// Standard ZX keyboard only in lower case. EMULATOR KEYMODE should use bNormal only.
		// Extra Modes for using keyboard as a normal(ish) PS2 / USB device.
		byte bNormalShifted;	// Red Symbols & Words (replaced by Symbols)  
		byte bGraphics;			// Standard ZX keyboard but should be used with CTL characters. Numbers = F1 - F10 etc
		byte bGraphicsShifted;	// Standard ZX keyboard but should be used with ALT characters. Numbers = F11 - F12, 5 = home, 6 PGUP etc 
		
		// Monitor Keys last press / activity.
		short iDebounceCount = DEBOUNCE_DELAY;
		short iDebounceCountHighest = iDebounceCount;

		byte KeyValue(KEYSTATES eKeystate) {
			// Get Keyboard character for correct Key state.
			byte bKeyValue = 0;

			switch (eKeystate) {
				case FUNCTION:
					bKeyValue = bNormal;
					// Adjust for Capital Letters
					if (bKeyValue > 96 && bKeyValue < 123){
						bKeyValue = bKeyValue - 32;
					}
					break;
					
				case NORMAL_SHIFTED:
					bKeyValue = bNormalShifted;
					break;
					
				case GRAPHICS:
					bKeyValue = bGraphics;
					break;
					
				case GRAPHICS_SHIFTED:
					bKeyValue = bGraphicsShifted;
					break;
					
				default:
					// Standard normal and Emulator mode keyboard.
					// Shift key modifier used later to select functions etc in a ZX81 Emulator.
					bKeyValue = bNormal;
					break;
			}
			return bKeyValue;
		}
		
	public:
		ZxKey (byte, byte, byte, byte);
			
		byte KeyPressDetected(KEYSTATES eKeystate, KEYMODES eKeymode){
			byte bKeyValue = 0;
			iDebounceCount--;
			if (iDebounceCount == 0){
				switch (iDebounceCountHighest) {
					case DEBOUNCE_DELAY:
						iDebounceCount = DEBOUNCE_DELAY_REPEAT;
						iDebounceCountHighest = iDebounceCount;
					break;
					
					case DEBOUNCE_DELAY_REPEAT:
						if (eKeymode == EMULATOR){
							iDebounceCount = DEBOUNCE_DELAY;
						} else {
							iDebounceCount = DEBOUNCE_DELAY / 2;
						}
					break;
					
					default:
						iDebounceCount = DEBOUNCE_DELAY;
					break;
				}
				bKeyValue = KeyValue(eKeystate);
			}
			return bKeyValue;
		}
		
		void KeyPressReset(){
			iDebounceCount = DEBOUNCE_DELAY;
			iDebounceCountHighest = iDebounceCount;
		}
	
		void KeyDisable(){
			iDebounceCount = -1;
		}
};


// ZxKey constructor
ZxKey::ZxKey (byte Normal, byte NormalShifted, byte Graphics, byte GraphicsShifted){
	bNormal = Normal;
	bNormalShifted = NormalShifted;
	bGraphics = Graphics;
	bGraphicsShifted = GraphicsShifted;
}


// Defines entire keyboard, includes ZxKey class.
class ZxKeyBoard {
	private:
		// Setup 4 versions of keyboard states into keyboard.
		// Keyboard mapped for US, might need changing for other configurations eg. UK keyboard.
		ZxKey keyMap[NUM_ROWS][NUM_COLS] = 
		{
			{{'5',KEY_LEFT_ARROW,KEY_F5,KEY_HOME},{'4','%',KEY_F4,KEY_INSERT},{'3','#',KEY_F3,KEY_ESC},{'2','@',KEY_F2,KEY_F12},{'1','!',KEY_F1,KEY_F11}},
			{{'t','_','t','t'},{'r','&','r','r'},{'e','^','e','e'},{'w','`','w','w'},{'q','~','q','q'}},
			{{'6',KEY_DOWN_ARROW,KEY_F6,KEY_PAGE_DOWN},{'7',KEY_UP_ARROW,KEY_F7,KEY_PAGE_UP},{'8',KEY_RIGHT_ARROW,KEY_F8,KEY_END},{'9','9',KEY_F9,'9'},{'0',KEY_BACKSPACE,KEY_F10,'0'}},
			{{'g','\\','g','g'},{'f','}','f','f'},{'d','{','d','d'},{'s',']','s','s'},{'a','[','a','a'}},
			{{'y','|','y','y'},{'u','$','u','u'},{'i','(','i','i'},{'o',')','o','o'},{'p','"','p','p'}},
			{{'v','/','v','v'},{'c','?','c','c'},{'x',';','x','x'},{'z',':','z','z'},{0,0,0,0}},
			{{'h','\'','h','h'},{'j','-','j','j'},{'k','+','k','k'},{'l','=','l','l'},{KEY_RETURN,KEY_RETURN,KEY_RETURN,KEY_RETURN}},
			{{'b','*','b','b'},{'n','<','n','n'},{'m','>','m','m'},{'.',',','.','.'},{' ',KEY_TAB,'£',' '}}
		};
		
	public:
		// Return from ZxKey matching specified state.
		byte bKeyPress(byte bRow, byte bCol, KEYSTATES eKeystate, KEYMODES eKeymode){
			return keyMap[bRow][bCol].KeyPressDetected(eKeystate, eKeymode);
		}
		
		void KeyPressReset(byte bRow, byte bCol){;
			keyMap[bRow][bCol].KeyPressReset();
		}
		
		void KeyDisable(byte bRow, byte bCol){;
			keyMap[bRow][bCol].KeyDisable();
		}
};
